@@ -231,7 +231,7 @@ impl Trainer {
 		Ok(trainer)
 	}
 
-	/// Train each model in the grid and compute model comparison metrics.
+	/// Train each model in the grid and compute comparison metrics.
 	pub fn train_grid(
 		&mut self,
 		kill_chip: &KillChip,
@@ -290,7 +290,7 @@ impl Trainer {
 
 		// Choose the best model.
 		let (train_model_output, best_grid_item_index) =
-			choose_best_model(&train_grid_item_outputs, &comparison_metric);
+			choose_best_model(&train_grid_item_outputs, &comparison_metric)?;
 
 		// Test the best model.
 		let test_metrics = test_model(&train_model_output, &table_test, &mut |progress_event| {
@@ -873,8 +873,8 @@ fn compute_baseline_metrics(
 
 pub struct TrainGridItemOutput {
 	pub train_model_output: TrainModelOutput,
-	pub model_comparison_metrics: Metrics,
-	pub model_comparison_metric_value: f32,
+	pub comparison_metrics: Metrics,
+	pub comparison_metric_value: f32,
 	pub duration: Duration,
 }
 
@@ -898,8 +898,8 @@ fn train_grid_item(
 		}))
 	});
 	let duration = start.elapsed();
-	let model_comparison_metrics =
-		compute_model_comparison_metrics(&train_model_output, &table_comparison, &mut |progress| {
+	let comparison_metrics =
+		compute_comparison_metrics(&train_model_output, &table_comparison, &mut |progress| {
 			handle_progress_event(ProgressEvent::Train(TrainProgressEvent {
 				grid_item_index,
 				grid_item_count,
@@ -908,20 +908,17 @@ fn train_grid_item(
 				),
 			}))
 		});
-	let model_comparison_metric_value =
-		get_model_comparison_metric_value(&model_comparison_metrics, comparison_metric);
+	let comparison_metric_value =
+		get_comparison_metric_value(&comparison_metrics, comparison_metric);
 	TrainGridItemOutput {
 		train_model_output,
-		model_comparison_metrics,
-		model_comparison_metric_value,
+		comparison_metrics,
+		comparison_metric_value,
 		duration,
 	}
 }
 
-fn get_model_comparison_metric_value(
-	metrics: &Metrics,
-	comparison_metric: ComparisonMetric,
-) -> f32 {
+fn get_comparison_metric_value(metrics: &Metrics, comparison_metric: ComparisonMetric) -> f32 {
 	match (comparison_metric, metrics) {
 		(ComparisonMetric::Regression(comparison_metric), Metrics::Regression(metrics)) => {
 			match comparison_metric {
@@ -1502,7 +1499,7 @@ fn choose_comparison_metric(config: &Config, task: &Task) -> Result<ComparisonMe
 						Ok(ComparisonMetric::Regression(RegressionComparisonMetric::R2))
 					}
 					metric => Err(anyhow!(
-						"{} is an invalid model comparison metric for regression",
+						"{} is an invalid comparison metric for regression",
 						metric
 					)),
 				}
@@ -1521,7 +1518,7 @@ fn choose_comparison_metric(config: &Config, task: &Task) -> Result<ComparisonMe
 						))
 					}
 					metric => Err(anyhow!(
-						"{} is an invalid model comparison metric for binary classification",
+						"{} is an invalid comparison metric for binary classification",
 						metric,
 					)),
 				}
@@ -1540,7 +1537,7 @@ fn choose_comparison_metric(config: &Config, task: &Task) -> Result<ComparisonMe
 						))
 					}
 					metric => Err(anyhow!(
-						"{} is an invalid model comparison metric for multiclass classification",
+						"{} is an invalid comparison metric for multiclass classification",
 						metric,
 					)),
 				}
@@ -1553,7 +1550,7 @@ fn choose_comparison_metric(config: &Config, task: &Task) -> Result<ComparisonMe
 	}
 }
 
-fn compute_model_comparison_metrics(
+fn compute_comparison_metrics(
 	train_model_output: &TrainModelOutput,
 	table_comparison: &TableView,
 	handle_progress_event: &mut dyn FnMut(ModelTestProgressEvent),
@@ -1661,7 +1658,7 @@ fn compute_model_comparison_metrics(
 fn choose_best_model(
 	outputs: &[TrainGridItemOutput],
 	comparison_metric: &ComparisonMetric,
-) -> (TrainModelOutput, usize) {
+) -> Result<(TrainModelOutput, usize)> {
 	match comparison_metric {
 		ComparisonMetric::Regression(comparison_metric) => {
 			choose_best_model_regression(outputs, comparison_metric)
@@ -1678,49 +1675,45 @@ fn choose_best_model(
 fn choose_best_model_regression(
 	outputs: &[TrainGridItemOutput],
 	comparison_metric: &RegressionComparisonMetric,
-) -> (TrainModelOutput, usize) {
+) -> Result<(TrainModelOutput, usize)> {
 	outputs
 		.iter()
 		.enumerate()
-		.max_by(|(_, output_a), (_, output_b)| {
-			let metrics_a = match &output_a.model_comparison_metrics {
+		.filter_map(|(index, output)| {
+			let metrics = match &output.comparison_metrics {
 				Metrics::Regression(metrics) => metrics,
 				_ => unreachable!(),
 			};
-			let metrics_b = match &output_b.model_comparison_metrics {
-				Metrics::Regression(metrics) => metrics,
-				_ => unreachable!(),
+			let metric = match comparison_metric {
+				RegressionComparisonMetric::MeanAbsoluteError => -metrics.mae,
+				RegressionComparisonMetric::RootMeanSquaredError => -metrics.rmse,
+				RegressionComparisonMetric::MeanSquaredError => -metrics.mse,
+				RegressionComparisonMetric::R2 => metrics.r2,
 			};
-			match comparison_metric {
-				RegressionComparisonMetric::MeanAbsoluteError => {
-					metrics_b.mae.partial_cmp(&metrics_a.mae).unwrap()
-				}
-				RegressionComparisonMetric::RootMeanSquaredError => {
-					metrics_b.rmse.partial_cmp(&metrics_a.rmse).unwrap()
-				}
-				RegressionComparisonMetric::MeanSquaredError => {
-					metrics_b.mse.partial_cmp(&metrics_a.mse).unwrap()
-				}
-				RegressionComparisonMetric::R2 => metrics_a.r2.partial_cmp(&metrics_b.r2).unwrap(),
+			if metric.is_finite() {
+				Some((index, output, metric))
+			} else {
+				None
 			}
 		})
-		.map(|(index, output)| (output.train_model_output.clone(), index))
-		.unwrap()
+		.max_by(|(_, _, metric_a), (_, _, metric_b)| metric_a.partial_cmp(metric_b).unwrap())
+		.ok_or_else(|| anyhow!("None of the models trained had a finite comparison metric value."))
+		.map(|(index, output, _)| (output.train_model_output.clone(), index))
 }
 
 fn choose_best_model_binary_classification(
 	outputs: &[TrainGridItemOutput],
 	comparison_metric: &BinaryClassificationComparisonMetric,
-) -> (TrainModelOutput, usize) {
-	outputs
+) -> Result<(TrainModelOutput, usize)> {
+	Ok(outputs
 		.iter()
 		.enumerate()
 		.max_by(|(_, output_a), (_, output_b)| {
-			let metrics_a = match &output_a.model_comparison_metrics {
+			let metrics_a = match &output_a.comparison_metrics {
 				Metrics::BinaryClassification(metrics) => metrics,
 				_ => unreachable!(),
 			};
-			let metrics_b = match &output_b.model_comparison_metrics {
+			let metrics_b = match &output_b.comparison_metrics {
 				Metrics::BinaryClassification(metrics) => metrics,
 				_ => unreachable!(),
 			};
@@ -1732,22 +1725,22 @@ fn choose_best_model_binary_classification(
 			}
 		})
 		.map(|(index, output)| (output.train_model_output.clone(), index))
-		.unwrap()
+		.unwrap())
 }
 
 fn choose_best_model_multiclass_classification(
 	outputs: &[TrainGridItemOutput],
 	comparison_metric: &MulticlassClassificationComparisonMetric,
-) -> (TrainModelOutput, usize) {
-	outputs
+) -> Result<(TrainModelOutput, usize)> {
+	Ok(outputs
 		.iter()
 		.enumerate()
 		.max_by(|(_, output_a), (_, output_b)| {
-			let metrics_a = match &output_a.model_comparison_metrics {
+			let metrics_a = match &output_a.comparison_metrics {
 				Metrics::MulticlassClassification(metrics) => metrics,
 				_ => unreachable!(),
 			};
-			let metrics_b = match &output_b.model_comparison_metrics {
+			let metrics_b = match &output_b.comparison_metrics {
 				Metrics::MulticlassClassification(metrics) => metrics,
 				_ => unreachable!(),
 			};
@@ -1758,7 +1751,7 @@ fn choose_best_model_multiclass_classification(
 			}
 		})
 		.map(|(index, output)| (output.train_model_output.clone(), index))
-		.unwrap()
+		.unwrap())
 }
 
 fn test_model(
